{------------------------------------------------------------------------------}
Unit Globals;
{------------------------------------------------------------------------------}
Interface
{------------------------------------------------------------------------------}
Uses Windows,Forms,OLE2,Dialogs,Sysutils,Math,
     D3D,D3DRM,D3DRMObj,D3DTypes,D3DRMDef,DDraw,D3DCaps;                        { Direct3D libraries }
{------------------------------------------------------------------------------}
Const
     Cx:Single=0;                                                               { Camera X,Y and Z positions }
     Cy:Single=0;
     Cz:Single=-70;
{------------------------------------------------------------------------------}
Var
   Direct3DRM:IDirect3DRM;                                                      { Globally Visible Objects }
   Device:IDirect3DRMDevice;
   Scene:IDirect3DRMFrame;
   Camera:IDirect3DRMFrame;
{------------------------------------------------------------------------------}
Procedure CreateScene;                                                          { Creates the 3D scene }
Procedure SetUpWindow(Form:TForm);                                              { Creates the Device,Camera and Viewport based on form }
Procedure Try3DRM(rval:HRESULT);                                                { Helper function to test if Direct3D call was successful }
Procedure SetupTextureColorValues(Form:TForm);                                  { Sets up the texture color values based on the color depth of the form }
{------------------------------------------------------------------------------}
Implementation
{------------------------------------------------------------------------------}
Var
   Light:IDirect3DRMFrame;                                                      { Internal Objects }
   Viewport:IDirect3DRMViewport;
   Clipper:IDirectDRAWClipper;
   Light1, Light2:IDirect3DRMLight;
   Frame:IDirect3DRMFrame;
{------------------------------------------------------------------------------}
Procedure Try3DRM(rval:HRESULT);
Begin
 If rVal <> D3DRM_OK                                                            { Raise Exception if result is not = to D3DRM_OK }
    Then Raise Exception.CreateFmt('Error Code:%x',[rVal]);
End;
{------------------------------------------------------------------------------}
Procedure SetupTextureColorValues(Form:TForm);
Var
   bpp:Integer;
Begin
 bpp := GetDeviceCaps(Form.Canvas.Handle,BITSPIXEL);                            { Get color depth of form }

 Case bpp of                                                                    { Setup Texture Shades and colors }
  1: Begin                                                                      { You may want to change these values to get better speed and rendering qualities }
      Direct3DRM.SetDefaultTextureShades(4);
     End;
 16: Begin
      Direct3DRM.SetDefaultTextureColors(256);
      Direct3DRM.SetDefaultTextureShades(256);
     End;
 24 or 32:Begin
           Direct3DRM.SetDefaultTextureColors(256);
           Direct3DRM.SetDefaultTextureShades(256);
          End;
 End;
End;
{------------------------------------------------------------------------------}
Procedure CreateScene;
Const
     ModelName='box.x';                                                         { Name of Model to Load }
Var
   Loop:Integer;                                                                { Loop for Adding random boxes }
   FrameNew:IDirect3DRMFrame;                                                   { Frame to hold copies of Box }
Begin
 Try3DRM(Direct3DRM.CreateFrame(Nil,Scene));                                    { Create Highlevel Frame. The Scene }

 Try3DRM(Direct3DRM.CreateLightRGB(D3DRMLIGHT_PARALLELPOINT,0.8, 0.8, 0.8, light1));       { Create Point Light }
 Try3DRM(Direct3DRM.CreateLightRGB(D3DRMLIGHT_AMBIENT, 0.5, 0.5, 0.5, light2)); { Create Ambient Light }

 Try3DRM(Direct3DRM.CreateFrame(Scene, light));                                 { Create Frame for Lights as child of Scene Frame }

 Try3DRM(Light.AddLight(light1));                                               { Add lights to Light Frame }
 Try3DRM(Light.AddLight(light2));

 light1.Release;                                                                { Free up lights. The Add to the light frame incremented references counts on the lights. So lights are not really destroyed }
 light2.Release;

 Try3DRM(Direct3DRM.CreateFrame(Scene, Frame));                                 { Create a new frame as a child of the Scene frame. }

 Try3DRM(Frame.Load(pChar(ModelName),Nil, D3DRMLOAD_FROMFILE,Nil,Nil));         { Load model from .X file :) }

 Try3DRM(Frame.SetPosition(Scene,0.0,0,0.0));                                   { Set position of Frame frame relative to Scene frame }

 Try3DRM(light.SetPosition(Scene, 0, 50, CZ*2));                                { Set position of Light frame relative to Scene frame }

 Try3DRM(Light.LookAt(Frame,Scene,D3DRMCONSTRAIN_Z));                           { Set Light frame to look at Frame frame, relative to Scene and constrined along the Z axis }

 Try3DRM(Frame.SetRotation(Nil,1,1,1,DegtoRad(5)));                             { Set frame to rotate around all axises(?) by 5 degrees }

 For Loop:=0 to 100 do                                                          { Adds 100 copies of Frame at random positions }
  Begin
   Try3DRM(Direct3DRM.CreateFrame(Scene, FrameNew));                            { Create a new child frame (FrameNew) to Scene }
   Try3DRM(FrameNew.AddVisual(Frame));                                          { Copy Frame into FrameNew }
   Try3DRM(FrameNew.SetPosition(Scene,Random(800)-400,Random(800)-400,Random(800)-400));        { Set position of Frame new to a random position }
   Try3DRM(FrameNew.SetRotation(Nil,Random(3)-2,Random(3)-2,Random(3)-2,DegtoRad(Random(15)))); { Set rotation of frame to random rotation }
  End;
End;
{------------------------------------------------------------------------------}
Function BppToDDBD(BPP:Integer):DWORD;                                          { Gets the DDDB constant from a BPP value }
Begin                                                                           { This is some code I got from a C++ demo that in turn got it from the DirectX SDK }
 Case BPP of
   1:Result:= DDBD_1;
   2:Result:= DDBD_2;
   4:Result:= DDBD_4;
   8:Result:= DDBD_8;
  16:Result:= DDBD_16;
  24:Result:= DDBD_24;
  32:Result:= DDBD_32;
 Else
  Result:= 0;
 End;
End;
{------------------------------------------------------------------------------}
Function FindDevice(cm:D3DCOLORMODEL;Var GUID:TGUID):Boolean;                   { This code tries to find the best Direct3D Device to use }
Var                                                                             { This is some code I got from a C++ demo that in turn got it from the DirectX SDK }
    DirectDraw:IDirectDraw;
    Direct3D:IDirect3D;
    Search:D3DFindDeviceSearch;
    FindResult:D3DFindDeviceResult;

    Error:HResult;

    ScreenHandle:HDC;
    BPP:integer;
Begin
 ScreenHandle:= GetDC(0);                                                       { Get handle of Screen }
 BPP:= GetDeviceCaps(ScreenHandle, BITSPIXEL);                                  { Get Color depth of screen }
 ReleaseDC(0, ScreenHandle);                                                    { Release screen handle }

 if (DirectDrawCreate(Nil, DirectDraw, Nil)) <> DD_OK                           { Create a DirectDraw Instance }
     Then Begin
           Result:=False;
           Exit;
          End;

 If (DirectDraw.QueryInterface(IID_IDirect3D, Direct3D)) <> DD_OK               { Create a Direct3D Instance from DirectDraw Object }
    Then Begin
          DirectDraw.Release;
          Result:=False;
          Exit;
         End;

 ZeroMemory(@Search,SizeOf(Search));                                            { Clear search record }
 Search.dwSize:=SizeOf(Search);                                                 { Set up Search Record }
 Search.dwFlags:=D3DFDS_COLORMODEL;                                             { Searching for a device with a color model that matches the one specified }

 If CM = D3DCOLOR_MONO
    Then search.dcmColorModel:=D3DCOLOR_MONO                                    { A RAMP colour model }
    Else search.dcmColorModel:=D3DCOLOR_RGB;                                    { A true RGB colour model }

 ZeroMemory(@FindResult,SizeOf(FindResult));                                    { Clear Result Record }
 FindResult.dwSize:=SizeOf(FindResult);                                         { Set up Result Record }

 Error:= Direct3D.FindDevice(Search, FindResult);                               { Find device that matches specified colour model }

 If Error = DD_OK                                                               { Find was Successful }
    Then Begin
          If (FindResult.ddHwDesc.dwFlags <> 0) And                             { Hardware device flags set ? }
             (Not ( (FindResult.ddHwDesc.dwDeviceRenderBitDepth And bppToddbd(bpp)) =bppToddbd(bpp)))    { Color depth does not match screen color depth }
             Then Begin
                   Search.dwFlags:= D3DFDS_HARDWARE;                            { New Search }
                   Search.bHardware:= FALSE;                                    { Look for non hardware device }

                   ZeroMemory(@FindResult,SizeOf(FindResult));                  { Set up Result }
                   FindResult.dwSize:=SizeOf(FindResult);

                   Error := Direct3D.FindDevice(search, FindResult);            { Find Matching software device }
                  End;
         End;

  Direct3D.Release;                                                             { Free Direct3D Object }
  DirectDraw.Release;                                                           { Free Direct Draw Object }

  If Error <> DD_OK                                                             { Did not find a matching Device }
     Then Result:=False                                                         { Return False }
     Else Begin
           Result:=True;                                                        { Found device, Return True }
           GUID:=FindResult.Guid;                                               { Return GUID of Device }
          End;
End;
{------------------------------------------------------------------------------}
Procedure SetUpWindow(Form:TForm);                                              { This setsup the device,Camera and Viewport based on the form passed to it }
Var
   BPP:Integer;                                                                 { Holds Forms Color Depth }
   GUID:TGUID;                                                                  { Hold GUID of best Direct3D Device to use }
Begin
 BPP:=GetDeviceCaps(Form.Canvas.Handle,BITSPIXEL);                              { Get color depth of form }

 Try3DRM(Direct3DRM.CreateFrame(Scene, Camera));                                { Create a new frame child frame (Camera) in Scene Frame }
 Try3DRM(Camera.SetPosition(Scene, cx, cy,cz));                                 { Set position of Camera frame }
 Try3DRM(Camera.LookAt(Frame,Scene,D3DRMCONSTRAIN_Z));                          { Set Camera frame to look at Frame frame in relation to Scene frame and constrained by Z axis } 

 Try3DRM(DirectDrawCreateClipper(0, Clipper, Nil));                             { Create a Clipper Object }
 Try3DRM(Clipper.SetHWnd(0, Form.Handle));                                      { Set window for Clipper to be form passed to procedure }

 If FindDevice(D3DCOLOR_MONO,GUID)                                              { Find a Device that supports the RAMP color model }
    Then Try3DRM(Direct3DRM.CreateDeviceFromClipper(clipper, @GUID, Form.ClientWidth, Form.Clientheight, device)) { Create device based on Clipper using device with found GUID }
    Else Try3DRM(Direct3DRM.CreateDeviceFromClipper(clipper, Nil, Form.ClientWidth, Form.Clientheight, device));  { Create device based on Clipper using default device }

 Try3DRM(Direct3DRM.CreateViewport(device, camera, 0, 0, Form.ClientWidth, Form.Clientheight, viewport)); { Create a viewport that is as big as the form }
 Try3DRM(viewport.SetBack(5000.0));                                             { Set back clipping depth to be 5000 units away }

                                                                                { You may want to change these values to get better speed and rendering qualities }
 Try3DRM(device.SetDither(FALSE));                                              { Set device to no dither }
 Try3DRM(device.SetQuality(D3DRMRENDER_PHONG));                                 { Set to phong quality }
 Try3DRM(device.SetTextureQuality(D3DRMTEXTURE_LINEAR));                        { Set texture quality to linear }

 Case BPP of
  1: Begin
      Device.SetShades(4);                                                      { Set shades used by RAMP colour model. based on color depth of screen }
     End;
 16: Begin
      Device.SetShades(64);
     End;
 24 or 32:Begin
           Device.SetShades(256);
          End;
 End;

End;
{------------------------------------------------------------------------------}
Initialization
  Randomize;                                                                    { Randomize random Number Generator }
  Direct3DRMCreate(Direct3DRM);                                                 { Create a Direct3DRM object }
Finalization
  Direct3DRM.Release;                                                           { Release Direct3DRM object }
End.
{------------------------------------------------------------------------------}
