{------------------------------------------------------------------------------}
Unit MainForm3D;
{------------------------------------------------------------------------------}
{ This application shows how to use Direct3D in Retained Mode and how to       }
{ render to a window.                                                          }
{                                                                              } 
{ This unit holds the the main form of the application. This form creates the  }
{ 3D scene,the device and the viewport using the functions in the Globals Unit.}
{                                                                              }
{ This form also creates a IDIRECT3DRMWINDEVICE which provides helper functions}
{ for rendering to a window.                                                   }
{                                                                              }
{ This application uses a timer to trigger the update of the scene. Timers are }
{ icky, event driven and inaccurate. They also have bad resolution which means }
{ that the update of the scene could probably be happening faster if I had used}
{ some other way of triggering the update.                                     }
{                                                                              }
{ On my P-200 with a Matrox Mystique 2mb card I only get 18 fps :(             }
{                                                                              }
{ You can move the camera around using the following keys :                    }
{   UP    - Look Up                                                            }
{   DN    - Look Down                                                          }
{   LEFT  - Look Left                                                          }
{   RIGHT - Look Right                                                         }
{                                                                              }
{ The ESC key can be used to exit the application                              }
{                                                                              }
{ All units starting with D are from Blake Stone's DirectX 3 Header conversion }
{ Projects. For the latest copy of this go to http://www.dkw.com/bstone        }
{                                                                              }
{ This and other code can be found at :                                        }
{ The Delphi Codex - http://www.geocities.com/SiliconValley/Pines/6133/        }
{                                                                              } 
{ This code can be freely used and distributed.                                }
{                                                                              }
{ Jonathan Ackerman 22 March 1997                                              }
{------------------------------------------------------------------------------}
{ Version   Date       Description                                             }
{ -------   ----       -----------                                             }
{ 1.0       22/03/97   First Release of Code                                   }
{------------------------------------------------------------------------------}
Interface
{------------------------------------------------------------------------------}
Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,ExtCtrls,
  StdCtrls,
  Globals,                                                                     { Holds scene and device Creation Functions }
  D3DRMWin;                                                                    { Holds IDirect3DRMWinDevice definition }
{------------------------------------------------------------------------------}
Type
  TfrmMainForm = Class(TForm)
                  Timer1: TTimer;
                  Procedure FormCreate(Sender: TObject);
                  Procedure FormShow(Sender: TObject);
                  Procedure FormClose(Sender: TObject; var Action: TCloseAction);
                  Procedure FormPaint(Sender: TObject);
                  Procedure FormActivate(Sender: TObject);
                  Procedure Timer1Timer(Sender: TObject);
                  Procedure FormKeyUp(Sender: TObject; var Key: Word;
                                      Shift: TShiftState);
                 Private
                  WinDev:IDirect3DRMWinDevice;                                 { Provides helper functions for rendering to a window }
                  UpInc,DnInc,LeftInc,RightInc:Double;                         { Used to hold Deltas for Camera movement. Helps to give smooth movement }
                  CXRot,CYRot:Double;                                          { Holds Camera Y and X Rotation Values }
                  FrameCount:Cardinal;                                         { Frame Counter. Incremented on every Frame. Used to Calc FPS. }
                  StartTick:DWORD;                                             { Clicktick of first frame. Used to Calc FPS}
                 End;
{------------------------------------------------------------------------------}
Var
  frmMainForm: TfrmMainForm;
{------------------------------------------------------------------------------}
Implementation
{------------------------------------------------------------------------------}
{$R *.DFM}
{------------------------------------------------------------------------------}
Uses D3DRMDef,                                                                  { Holds type definitions }
     D3DTypes,
     Math;                                                                      { Holds DegToRad Function }
{------------------------------------------------------------------------------}
Procedure TfrmMainForm.FormCreate(Sender: TObject);
Begin
 Screen.Cursor:=crNone;
 Try
  SetupTextureColorValues(Self);                                                { These values need to be setup before any textures are created, so call is before call to CreateScene}
  CreateScene;                                                                  { Call to Create the 3D scene. }
 Except
  Halt;                                                                         { Assume any error is fatal so halt. }
 End;
End;

Procedure TfrmMainForm.FormShow(Sender: TObject);
Begin
 Try
  SetUpWindow(Self);                                                            { Setup devices, camera and viewport }
  Try3DRM(device.QueryInterface(IID_IDirect3DRMWinDevice, windev));             { Create IDirect3DRMWinDevice }
 Except
  Halt;                                                                         { Assume any error is fatal so halt. }
 End;
End;

Procedure TfrmMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
Begin
 Windev.Release;                                                                { Freeup IDirect3DRMWinDevice }
End;

Procedure TfrmMainForm.FormPaint(Sender: TObject);
Begin
 Try3DRM(windev.HandlePaint(Canvas.Handle));                                    { Use IDirect3DRMWinDevice to handle window paint }
End;

Procedure TfrmMainForm.FormActivate(Sender: TObject);
Begin
 Try3DRM(windev.HandleActivate(0));                                             { Use IDirect3DRMWinDevice to handle repaint when application is activated }
End;

procedure TfrmMainForm.Timer1Timer(Sender: TObject);
Const
    Step=1;                                                                     { Step of Delta Increase }
    DownStep=0.25;
    MaxDelta=10;                                                                { Maximum Delta }
Var
    CurrentTick:DWORD;                                                          { Used in fps calcs. }
Begin
 If GetAsyncKeyState(VK_UP) <> 0                                                { Check if UP button is being held down }
    Then UpInc:=UpInc+Step                                                      { If it is down increment delta }
    Else UpInc:=UpInc-DownStep;                                                 { else decrement delta }
 If UpInc < 0                                                                   { Check that delta does not exceed limits}
    Then UpInc:=0;
 If UpInc > MaxDelta
    Then UpInc:=MaxDelta;

 If GetAsyncKeyState(VK_DOWN) <> 0                                              { As above but for DOWN button }
    Then DnInc:=DnInc+Step
    Else DNInc:=DNInc-DownStep;
 If DNInc < 0
    Then DNInc:=0;
 If UpInc > MaxDelta
    Then DNInc:=MaxDelta;

 CXRot:=CxRot-UpInc+DnInc;                                                      { Calculate X Rotation }

 If GetAsyncKeyState(VK_LEFT) <> 0                                              { As above but for LEFT key }
    Then LeftInc:=LeftInc+Step
    Else LeftInc:=LeftInc-DownStep;
 If LeftInc < 0
    Then LeftInc:=0;
 If LeftInc > MaxDelta
    Then LeftInc:=MaxDelta;

 If GetAsyncKeyState(VK_RIGHT) <> 0                                             { As above but for RIGHT key }
    Then RightInc:=RightInc+Step
    Else RightInc:=RightInc-DownStep;
 If RightInc < 0
    Then RightInc:=0;
 If RightInc > MaxDelta
    Then RightInc:=MaxDelta;

 CYRot:=CYRot-LeftInc+RightInc;                                                 { Calculate the Y Rotation }

 Camera.AddRotation(D3DRMCOMBINE_REPLACE,1,0,0,DegToRad(CxRot));                { Set Camera Rotation }
 Camera.AddRotation(D3DRMCOMBINE_AFTER,0,1,0,DegToRad(CyRot));

 Direct3DRM.Tick(1);                                                            { Do Tick. This updates animations and rerenders scene }
 Inc(FrameCount);                                                               { Increment Frame Counter }

 CurrentTick:=GetTickCount;                                                     { Get Current Clock Tick }
 If (CurrentTick-StartTick) > 1000                                              { Has a second elapsed ? }
    Then Begin
          Caption:='Test Direct3D - '+                                          { Set Form Caption to fps value }
                    IntToStr(Round(FrameCount))+' fps';
          StartTick:=GetTickCount;                                              { Reset Start }
          FrameCount:=0;                                                        { Reset Frame Counter }
         End;
End;

Procedure TfrmMainForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
Begin
 If Key=VK_ESCAPE                                                               { If Escape pressed then Close Application }
    Then Close;
End;

End.
{------------------------------------------------------------------------------}
